#include <libtools.h>
#include <anymap.h>

typedef std::function<int(com::Anymap)> IntFunc;
typedef std::function<bool(com::Anymap)> BoolFunc;
typedef std::function<bool*(com::Anymap)> BoolArrFunc;  

IntFunc LoadEntry(std::string lib, std::string func) {
    DllHandle handler;
    return com::lib::loadFunction<int (com::Anymap), DllHandle>(lib.c_str(), func.c_str(), handler);
}

BoolFunc LoadPred(std::string lib, std::string func) {
    DllHandle handler;
    return com::lib::loadFunction<int (com::Anymap), DllHandle>(lib.c_str(), func.c_str(), handler);
}

BoolArrFunc LoadSelector(std::string lib, std::string func){
    DllHandle handler;
    return com::lib::loadFunction<bool* (com::Anymap), DllHandle>(lib.c_str(), func.c_str(), handler);
}

void check_pred(bool predval, std::string predname) {
    if predval{
        std::cout<<"Predicate"<<predname<<"returned FALSE!"<<std::endl;
        exit(-1)
    }   
}

int main(int argc, char const *argv[])
{   
    auto data = com::Anymap();
    //Predicates
    auto test_funcs.simplest_positiveness_predicate = LoadPred("test_funcs.simplest", "positiveness_predicate"); 
    auto libcomsdk_pass_predicate = LoadPred("libcomsdk", "pass_predicate"); 
    //Entry functions
    auto test_funcs.simplest_increment_a_edge = LoadEntry("test_funcs.simplest", "increment_a_edge"); 
    auto libgcdfes_pso_swarm_init = LoadEntry("libgcdfes", "pso_swarm_init"); 
    auto libgcdfes_pso_agent_init = LoadEntry("libgcdfes", "pso_agent_init"); 
    auto libgcdfes_pso_task_data_reinit = LoadEntry("libgcdfes", "pso_task_data_reinit"); 
    auto libgcdfes_pso_hom_agent_postproc = LoadEntry("libgcdfes", "pso_hom_agent_postproc"); 
    auto libgcdfes_pso_agent_reinit = LoadEntry("libgcdfes", "pso_agent_reinit"); 
    auto libgcdfes_pso_swarm_analysing = LoadEntry("libgcdfes", "pso_swarm_analysing"); 
    auto libgcdfes_pso_swarm_reinit = LoadEntry("libgcdfes", "pso_swarm_reinit"); 
    auto libcomsdk_pass_processor = LoadEntry("libcomsdk", "pass_processor"); 
    //Selectors
    //Branch tokens
    bool* SEL___BEGIN__ = new bool[1];
    std::fill_n(SEL___BEGIN__, 1, true);
    bool* SEL_gcdhom_inverted_model_pso1_S_4 = new bool[1];
    std::fill_n(SEL_gcdhom_inverted_model_pso1_S_4, 1, false);
    bool* SEL_gcdhom_inverted_model_pso1_S_6 = new bool[2];
    std::fill_n(SEL_gcdhom_inverted_model_pso1_S_6, 2, false);
    bool* SEL_gcdhom_inverted_model_pso1_S_7 = new bool[2];
    std::fill_n(SEL_gcdhom_inverted_model_pso1_S_7, 2, false);
    bool* SEL___END__ = new bool[0];
    std::fill_n(SEL___END__, 0, false);

if (SEL___BEGIN__[0]){
	check_pred(test_funcs.simplest_positiveness_predicate(data), "test_funcs.simplest_positiveness_predicate");
	test_funcs.simplest_increment_a_edge(data);
	check_pred(libcomsdk_pass_predicate(data), "libcomsdk_pass_predicate");
	libgcdfes_pso_swarm_init(data);//Инициализация метода роя частиц.
	check_pred(libcomsdk_pass_predicate(data), "libcomsdk_pass_predicate");
	libgcdfes_pso_swarm_init(data);//Инициализация роя частиц.
	check_pred(libcomsdk_pass_predicate(data), "libcomsdk_pass_predicate");
	libgcdfes_pso_agent_init(data);//Инициализация отдельной частицы в рамках метода роя частиц.
}
gcdhom_inverted_model_pso1_S_4:
if (!(false || SEL___BEGIN__[0] || SEL_gcdhom_inverted_model_pso1_S_6[0] || SEL_gcdhom_inverted_model_pso1_S_7[0] )){
	for (int seli = 0; seli < 1; seli++)
		SEL_gcdhom_inverted_model_pso1_S_4[seli]=false;
}else {
	for (int seli = 0; seli < 1; seli++)
		SEL_gcdhom_inverted_model_pso1_S_4[seli]=true;
}
if (SEL_gcdhom_inverted_model_pso1_S_4[0]){
	check_pred(libcomsdk_pass_predicate(data), "libcomsdk_pass_predicate");
	libgcdfes_pso_task_data_reinit(data);//Реинициализация постановки задачи анализа эффективных характеристик КМ методом асимптотического осреднения.
	check_pred(libcomsdk_pass_predicate(data), "libcomsdk_pass_predicate");
	libgcdfes_pso_hom_agent_postproc(data);//Постпроцессинг после решения отдельной задачи методом асимптотического осреднения.
}
gcdhom_inverted_model_pso1_S_6:
if (!(false || SEL_gcdhom_inverted_model_pso1_S_4[0] )){
	for (int seli = 0; seli < 2; seli++)
		SEL_gcdhom_inverted_model_pso1_S_6[seli]=false;
}else {
	for (int seli = 0; seli < 2; seli++)
		SEL_gcdhom_inverted_model_pso1_S_6[seli]=true;
}
if (SEL_gcdhom_inverted_model_pso1_S_6[0]){
	check_pred(libcomsdk_pass_predicate(data), "libcomsdk_pass_predicate");
	libgcdfes_pso_agent_reinit(data);//Реинициализация отдельной частицы (смещение частицы) в рое в рамках метода роя частиц.
	goto gcdhom_inverted_model_pso1_S_4;
}
if (SEL_gcdhom_inverted_model_pso1_S_6[1]){
	check_pred(libcomsdk_pass_predicate(data), "libcomsdk_pass_predicate");
	libgcdfes_pso_swarm_analysing(data);//Анализ всего роя частиц в рамках метода роя частиц.
}
gcdhom_inverted_model_pso1_S_7:
if (!(false || SEL_gcdhom_inverted_model_pso1_S_6[1] )){
	for (int seli = 0; seli < 2; seli++)
		SEL_gcdhom_inverted_model_pso1_S_7[seli]=false;
}else {
	for (int seli = 0; seli < 2; seli++)
		SEL_gcdhom_inverted_model_pso1_S_7[seli]=true;
}
if (SEL_gcdhom_inverted_model_pso1_S_7[0]){
	check_pred(libcomsdk_pass_predicate(data), "libcomsdk_pass_predicate");
	libgcdfes_pso_swarm_reinit(data);//Реинициализация всего роя частиц в рамках метода роя частиц.
	goto gcdhom_inverted_model_pso1_S_4;
}
if (SEL_gcdhom_inverted_model_pso1_S_7[1]){
	//ПАСС, морфизм.
	check_pred(libcomsdk_pass_predicate(data), "libcomsdk_pass_predicate");
	libcomsdk_pass_processor(data);
}
if (!(false || SEL_gcdhom_inverted_model_pso1_S_7[1] )){
	for (int seli = 0; seli < 0; seli++)
		SEL___END__[seli]=false;
}else {
	for (int seli = 0; seli < 0; seli++)
		SEL___END__[seli]=true;
}


TERM:
    std::cout<<"Termination!\n"; 
    return 0;
}
